@@ -783,10 +783,10 @@ is_valid_keyname (const gchar  *key,
       return FALSE;
     }
 
-  if (i > 32)
+  if (i > 1024)
     {
       g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
-                   _("invalid name '%s': maximum length is 32"), key);
+                   _("invalid name '%s': maximum length is 1024"), key);
       return FALSE;
     }
 
@@ -1947,11 +1947,16 @@ main (int argc, char **argv)
     { NULL }
   };
 
-  setlocale (LC_ALL, "");
-  textdomain (GETTEXT_PACKAGE);
 #ifdef G_OS_WIN32
   extern gchar *_glib_get_locale_dir (void);
-  gchar *tmp = _glib_get_locale_dir ();
+  gchar *tmp;
+#endif
+
+  setlocale (LC_ALL, "");
+  textdomain (GETTEXT_PACKAGE);
+
+#ifdef G_OS_WIN32
+  tmp = _glib_get_locale_dir ();
   bindtextdomain (GETTEXT_PACKAGE, tmp);
   g_free (tmp);
 #else
@@ -2020,13 +2025,13 @@ main (int argc, char **argv)
 
       if (files->len == 0)
         {
-          fprintf (stderr, _("No schema files found: "));
+          fprintf (stdout, _("No schema files found: "));
 
           if (g_unlink (target))
-            fprintf (stderr, _("doing nothing.\n"));
+            fprintf (stdout, _("doing nothing.\n"));
 
           else
-            fprintf (stderr, _("removed existing output file.\n"));
+            fprintf (stdout, _("removed existing output file.\n"));
 
           return 0;
         }
